/* $Header: minopt_config.h,v 3.2 98/08/13 17:18:22 carl Exp $ */

/************************************************************************
 * Copyright (c) 1998 The Trustees of Princeton University.
 * All rights reserved.
 * 
 * PRINCETON UNIVERSITY MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 * OR IMPLIED WARRANTY.  PRINCETON UNIVERSITY SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY THE USERS OF THIS SOFTWARE.
 ************************************************************************/


#ifndef _MINOPT_CONFIG_INCLUDED
#define _MINOPT_CONFIG_INCLUDED

#include <float.h>

/********************  VERSION INFORMATION **********************/

#define VERSION "3.1"

/*************************** Constants *********************************/

#define MAXLINESIZE     80                      /* Maximum line size */

#define INFINITY        DBL_MAX
#define SMALLNUM        1e-12                   /* small number */

/* Problem identifiers */
#define LP              1                       /* LP problem */
#define MILP            2                       /* MILP problem */
#define NLP             3                       /* NLP problem */
#define OCP             4                       /* OCP problem */
#define MINLP           5                       /* MINLP problem */
#define RELAX           6                       /* relax problem */
#define PRIMAL          7                       /* primal problem */
#define IPRIMAL         8                       /* infeasible primal */
#define ALTIPRIMAL      9                       /* alternative infeasible primal */
#define MASTER          10                      /* master problem */
#define GBDMASTER       11                      /* GBD master problem */
#define OAERMASTER      12                      /* OAER master problem */
#define OAERAPMASTER    13                      /* AP master problem */
#define DMASTER         14                      /* DZW master problem */
#define DSUB1           15                      /* dual sub type 1 */
#define DSUB2           16                      /* dual sub type 2 */
#define DYNAMIC         17                      /* dynamic problem */
#define ADJOINT         18                      /* adjoint problem */

/* Different possibilities for the rows and overall problem */
/* Linear, Mixed-Integer Linear, Nonlinear, Separable */
#define L_TYPE          1      /* Linear */
#define N_TYPE          2      /* Nonlinear */
#define LN_TYPE         3      /* Linear/Nonlinear for aBB--ask Claire about this */
#define I_TYPE          4      /* Integer/Binary */
#define M_TYPE          5      /* Mixed Integer Linear */
#define P_TYPE          6      /* Mixed Integer Nonlinear */
#define O_TYPE          8      /* Optimal control */
#define S_TYPE          1      /* Separable */

/***************************** Problem type ****************************/
/* This option uses a long int field.  The right most four for the     *
 * master, the next four for the primal, and the next one for the      *
 * separability.  In binary, bits from right to left denote            *
 * linear, nonlinear, integer/binary.  (ie. SPPPPMMMM)                 */
 
#define PMASK_M         0x0000000f     /* master problem-type mask */
#define PMASK_P         0x000000f0     /* primal problem-type mask */
#define PMASK_S         0x00000100     /* separability mask */

/* determine the type of the problem */
#define PTYPE_M(p)      ((p) & PMASK_M)
#define PTYPE_P(p)      ((p) & PMASK_P) >> 4
#define PTYPE_S(p)      ((p) & PMASK_S)

/********************** Row type ***************************************/
/* Specifications for each row                                         */
/* Here we have a 12 bit field:  BSRD U Z Y X                          *
 * This denotes the Separability, Relaxation, Dual row, and the        *
 * linearity of the U,Z,X, and Y variables.  1 denotes true for SRDB,  *
 * and the linearity is 01 for linear, and 10 nonlinear.               */

#define RMASK_X         0x0003   /* mask for x type */
#define RMASK_Y         0x000C   /* mask for y type */
#define RMASK_Z         0x0030   /* mask for z type */
#define RMASK_U         0x00C0   /* mask for z type */
#define RMASK_D         0x0100   /* Dual row */
#define RMASK_R         0x0200   /* if row is not to be relaxed in infeasible primal */
#define RMASK_S         0x0400   /* separable in x and y */
#define RMASK_B         0x0800   /* row contains bilinear terms */

/* determine the type of row */
#define RTYPE_X(t)      ((t) & RMASK_X)
#define RTYPE_Y(t)      ((t) & RMASK_Y) >> 2
#define RTYPE_Z(t)      ((t) & RMASK_Z) >> 4
#define RTYPE_U(t)      ((t) & RMASK_U) >> 6
#define RTYPE_D(t)      ((t) & RMASK_D)
#define RTYPE_R(t)      ((t) & RMASK_R)
#define RTYPE_S(t)      ((t) & RMASK_S)
#define RTYPE_B(t)      ((t) & RMASK_B)

/* prototypes */
#if defined(__STDC__) || defined(__cplusplus)
# define P_(s) s
#else
# define P_(s) ()
#endif

char *get_problem_id_string P_((int type));
char *get_solver_status_string P_((int status));

#undef P_

#endif /* _MINOPT_CONFIG_INCLUDED */
