/* $Header: solutions.h,v 3.2 98/08/13 17:18:27 carl Exp $ */

/************************************************************************
 * Copyright (c) 1998 The Trustees of Princeton University.
 * All rights reserved.
 * 
 * PRINCETON UNIVERSITY MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 * OR IMPLIED WARRANTY.  PRINCETON UNIVERSITY SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY THE USERS OF THIS SOFTWARE.
 ************************************************************************/


#ifndef _SOLUTIONS_INCLUDED
#define _SOLUTIONS_INCLUDED

/* Contains solution structures for the different problems */

typedef struct optimalsol {     /* MINOPT Optimal solution */
  int    type;                  /* type of problem */
  int    solver;                /* solver used to solve this problem */
  int    status;                /* solution status */
  int    solverstatus;          /* specific status code returned from the solver */
  double ptime;                 /* time spent in primal problems */
  double mtime;                 /* time spent in master problems */
  double gtime;                 /* time required for generation */
  double stime;                 /* time required for solution */
  double ttime;                 /* total time required */
  int    iter;                  /* Iteration of solution */
  int    numiter;               /* number of iterations */
  int    numnode;               /* number of nodes (if MILP) */
  double objval;                /* Objective value */
  double *x;                    /* Solution for X */
  double *y;                    /* Solution for Y */
  struct dynamicsol *dynsol;    /* Solution for Z */
} *OPTSOLTYPE;

typedef struct primalsol {      /* Solution of primal problem */
                                /* Constraints are ordered Linear then Nonlinear */
  int    type;                  /* type of problem */
  int    solver;                /* solver used to solve this problem */
  int    solverstatus;          /* specific status code returned from the solver */
  int    status;                /* Solution status */
  double gtime;                 /* time required for generation */
  double stime;                 /* time required for solution */
  double ttime;                 /* total time required */
  int    numiter;               /* number of iterations */
  int    numnode;               /* number of nodes (if MILP) */
  double objval;                /* Objective value */
  double *yk;                   /* Fixed values for Y */
  double *x;                    /* Solution for X */
  double *cmargval;             /* Marginals for constraints */
  double *bmargval;             /* Marginals for bounds */
  double *dcdxval;              /* Values of the derivatives (Jacobian) */
  struct dynamicsol *dynsol;    /* Dynamic solution for this primal */
  struct adjointinfo *adjinfo;  /* Adjoint information for this primal */
  struct primalsol *next;       /* Pointer to next solution */
} *PRISOLTYPE;

typedef struct mastersol {      /* Solution of dual problem */
  int    type;                  /* type of problem */
  int    solver;                /* solver used to solve this problem */
  int    solverstatus;          /* specific status code returned from the solver */
  int    status;                /* status of solution */
  double gtime;                 /* time required for generation */
  double stime;                 /* time required for solution */
  double ttime;                 /* total time required */
  int    numiter;               /* number of iterations */
  int    numnode;               /* number of nodes (if MILP) */
  double objval;                /* Objective value */
  double *x;                    /* Solution for X */
  double *y;                    /* Solution for Y */
  double *z;                    /* Solution for Z */
  struct mastersol *next;       /* Pointer to next solution */
} *MASSOLTYPE;

typedef struct dynamicsol {     /* Dynamic solution from the integration */
  int    type;                  /* type of problem */
  int    solver;                /* solver used to solve this problem */
  int    solverstatus;          /* specific status code returned from the solver */
  int    status;                /* status of solution */
  double stime;                 /* time required for soution */
  int    neq;                   /* dimension of DAE system */
  int    npoints;               /* number of points in the dynamic solution */
  double *time;                 /* times for the values of the z variables */
  double **z;                   /* values of z and zdot variables at time instances */
  double **dzdx;                /* derivatives of z with respect to parameters */
} *DYNSOLTYPE;

typedef struct adjointinfo {    /* Adjoint information for the master problem */
  double *jacval;               /* jacobian values for the master problem */
  double rhs;                   /* right hand side values for the master problem */
} *ADJINFOTYPE;

#endif /* _SOLUTIONS_INCLUDED */
